@@ -33,7 +33,7 @@
 (declare-function telega-chatbuf-input-insert "telega-chat" (imc))
 (declare-function telega-chatbuf-attach-inline-bot-query "telega-chat" (&optional no-empty-search))
 (declare-function telega-chat--pop-to-buffer "telega-chat" (chat))
-(declare-function telega-chat-private-p "telega-chat" (chat &optional include-bots-p))
+(declare-function telega-chat-private-p "telega-chat" (chat))
 
 (defvar telega--inline-bot nil
   "BOT value for the inline results help buffer.")
@@ -46,32 +46,40 @@
 (defun telega--on-callbackQueryAnswer (reply)
   "Handle callback reply answer."
   (let ((text (telega-tl-str reply :text))
-        (link (plist-get reply :url)))
+        (link (telega-tl-str reply :url)))
     (if (plist-get reply :show_alert)
         ;; Popup message from the bot
         (with-telega-help-win "*Callback Alert*"
           (telega-ins text)
-          (unless (string-empty-p link)
+          (when link
             (telega-ins "\n")
-            (telega-ins--raw-button (telega-link-props 'url link)
+            (telega-ins--raw-button (telega-link-props 'url link 'face 'link)
               (telega-ins link))))
 
       (message text)
-      (unless (string-empty-p link)
+      (when link
         (telega-browse-url link)))))
 
-(defun telega--getCallbackQueryAnswer (msg payload)
+(cl-defun telega--getCallbackQueryAnswer (msg payload &key (callback #'ignore))
   "Async send callback to bot."
-  (telega-server--send
+  (declare (indent 2))
+  (telega-server--call
    (list :@type "getCallbackQueryAnswer"
          :chat_id (plist-get msg :chat_id)
          :message_id (plist-get msg :id)
-         :payload payload)))
+         :payload payload)
+   callback))
 
 (defun telega-inline--callback (kbd-button msg)
   "Action to take when KBD-BUTTON is pressed."
   (let ((kbd-type (plist-get kbd-button :type)))
     (cl-ecase (telega--tl-type kbd-type)
+      (keyboardButtonTypeWebApp
+       (telega-browse-url
+        (telega--getWebAppUrl (telega-msg-sender msg)
+          :url (telega-tl-str kbd-type :url))
+        'in-browser))
+
       (keyboardButtonTypeText
        ;; A simple button, with text that should be sent
        ;; when the button is pressed
@@ -82,19 +90,21 @@
              (imc (list :@type "inputMessageText"
                         :text (telega-string-fmt-text
                                (telega-tl-str kbd-button :text))))
-             (reply-msg (unless (telega-chat-private-p
-                                 (telega-msg-chat msg) 'bots)
+             (reply-msg (unless (telega-chat-private-p (telega-msg-chat msg))
                           msg)))
          (telega--sendMessage chat imc reply-msg)))
 
+      (keyboardButtonTypeRequestPhoneNumber
+       (telega-chat-share-my-contact (telega-msg-chat msg)))
+
       (inlineKeyboardButtonTypeUrl
-       (telega-browse-url (plist-get kbd-type :url)))
+       (telega-browse-url (telega-tl-str kbd-type :url)))
 
       (inlineKeyboardButtonTypeLoginUrl
        (let* ((url-info (telega--getLoginUrlInfo msg kbd-type))
               (url (or (plist-get url-info :url)
                        (plist-get kbd-type :url))))
-         (when (or (plist-get url-info :skip_confirm)
+         (when (or (plist-get url-info :skip_confirmation)
                    (not (memq telega-inline-login-url-action
                               '(query-all query-open)))
                    (y-or-n-p
@@ -111,8 +121,8 @@
                                  :domain (propertize
                                           (telega-tl-str url-info :domain)
                                           'face 'bold)
-                                 :user (telega-user--name
-                                        (telega-user-get telega--me-id)))
+                                 :user (telega-msg-sender-title (telega-user-me)
+                                         :with-username-p t))
                                "? ")))
              (let ((write-p (when (and (memq telega-inline-login-url-action
                                              '(query-all
@@ -121,10 +131,11 @@
                               (y-or-n-p
                                (concat (telega-i18n "lng_url_auth_allow_messages"
                                          :bot (propertize
-                                               (telega-user--name
+                                               (telega-msg-sender-title
                                                 (telega-user-get
                                                  (plist-get
-                                                  url-info :bot_user_id)))
+                                                  url-info :bot_user_id))
+                                                :with-username-p t)
                                                'face 'bold))
                                        "? ")))))
                (setq url (plist-get (telega--getLoginUrl msg kbd-type write-p)
@@ -151,7 +162,8 @@
 
            (telega-chatbuf--input-delete)
            (telega-chatbuf-input-insert
-            (concat "@" (telega-tl-str bot :username) " " (or new-query "")))
+            (concat (telega-msg-sender-username bot 'with-@)
+                    " " (or new-query "")))
            (telega-chatbuf-attach-inline-bot-query 'no-search))))
 
       (inlineKeyboardButtonTypeCallbackGame
@@ -168,6 +180,68 @@
        (when-let ((user (telega-user-get (plist-get kbd-type :user_id))))
          (telega-chat-with user)))
 
+      (inlineKeyboardButtonTypeWebApp
+       (let ((web-app-info (telega--openWebApp
+                            (telega-msg-chat msg)
+                            (telega-msg-sender msg)
+                            (plist-get kbd-type :url))))
+            (message "Webapp info: %S" web-app-info)
+            (telega-browse-url (plist-get web-app-info :url) 'in-browser)
+         ))
+
+      (keyboardButtonTypeRequestUsers
+       (let* ((user-temex
+               `(and ,(or telega-user-completing-temex
+                          '(return t))
+                     ,(if (plist-get kbd-type :restrict_user_is_bot)
+                          (if (plist-get kbd-type :user_is_bot)
+                              'is-bot
+                            '(not is-bot))
+                        '(return t))
+                     ,(if (plist-get kbd-type :restrict_user_is_premium)
+                          (if (plist-get kbd-type :user_is_premium)
+                              'is-premium
+                            '(not is-premium))
+                        '(return t))))
+              (users (telega-completing-read-user-list
+                         (telega-tl-str kbd-button :text)
+                       (telega-user-list user-temex))))
+         (when users
+           (telega--shareUsersWithBot msg (plist-get kbd-type :id) users))))
+
+      (keyboardButtonTypeRequestChat
+       (let* ((chat-temex
+               `(and (or is-known has-chatbuf)
+                     (not (type private))
+                     ,(if (plist-get kbd-type :chat_is_channel)
+                          '(type channel)
+                        '(not (type channel)))
+                     ,(if (plist-get kbd-type :restrict_chat_is_forum)
+                          (if (plist-get kbd-type :chat_is_forum)
+                              'is-forum
+                            '(not is-forum))
+                        '(return t))
+                     ,(if (plist-get kbd-type :restrict_chat_has_username)
+                          (if (plist-get kbd-type :chat_is_forum)
+                              'has-username
+                            '(not has-username))
+                        '(return t))
+                     ,(if (plist-get kbd-type :chat_is_created)
+                          'me-is-owner
+                        '(return t))
+                     ;; TODO:
+                     ;;   :user_administrator_rights
+                     ;;   :bot_administrator_rights
+                     ;;   :bot_is_member
+                     ))
+              (chat
+               (telega-completing-read-chat
+                (concat (telega-tl-str kbd-button :text) ": ")
+                (telega-filter-chats (telega-chats-list)
+                  chat-temex))))
+         (when chat
+           (telega--shareChatWithBot msg (plist-get kbd-type :id) chat))))
+
       ;; TODO: other types
       )))
 
@@ -176,7 +250,7 @@
   (let ((kbd-type (plist-get kbd-button :type)))
     (cl-case (telega--tl-type kbd-type)
       ((inlineKeyboardButtonTypeUrl inlineKeyboardButtonTypeLoginUrl)
-       (plist-get kbd-type :url))
+       (telega-tl-str kbd-type :url))
       )))
 
 
@@ -199,8 +273,8 @@
 (defun telega-ins--inline-delim ()
   "Inserter for the delimiter."
   (telega-ins--with-props
-      '(face default display ((space-width 2) (height 0.5)))
-    (telega-ins (make-string 30 ?─) "\n")))
+      '(face underline display ((space-width 2) (height 0.5)))
+    (telega-ins (make-string 30 ?\s) "\n")))
 
 (defun telega-inline-bot--action (qr)
   "Action to take when corresponding query result QR button is pressed."
@@ -231,10 +305,10 @@
            (thumb-file (when thumb (telega-file--renew thumb :file)))
            (thumb-img (when (telega-file--downloaded-p thumb-file)
                         (telega-create-image
-                         (telega--tl-get thumb-file :local :path)
-                         (when (fboundp 'imagemagick-types) 'imagemagick) nil
-                         :scale 1.0 :ascent 'center
-                         :height (telega-chars-xheight 1)))))
+                            (telega-file--path thumb-file) nil nil
+                          :scale 1.0
+                          :ascent 'center
+                          :height (telega-ch-height 1)))))
       (with-telega-chatbuf chat
         (telega-chatbuf--input-delete)
         (telega-chatbuf-input-insert
@@ -250,7 +324,7 @@
 (defun telega-ins--inline-audio (qr)
   "Inserter for `inlineQueryResultAudio' QR."
   (let ((audio (plist-get qr :audio)))
-    (telega-ins--audio nil audio telega-symbol-audio)
+    (telega-ins--audio nil audio 'full (telega-symbol 'audio))
     (telega-ins "\n")))
 
 (defun telega-ins--inline-voice-note (qr)
@@ -367,7 +441,7 @@
     (when thumb-img
       (telega-ins--image thumb-img 1)
       (telega-ins " "))
-    (telega-ins--with-face 'shadow
+    (telega-ins--with-face 'telega-shadow
       (telega-ins (telega-tl-str venue :address)))
     (telega-ins "\n")
     (when thumb-img
@@ -402,7 +476,7 @@
                             inlineQueryResultDocument
                             inlineQueryResultGame
                             inlineQueryResultVenue))
-                (unless (or (= (point) (point-at-bol))
+                (unless (or (= (point) (line-beginning-position))
                             (= (point) 1))
                   (telega-ins "\n")
                   (telega-ins--inline-delim)))
@@ -412,49 +486,48 @@
                  (telega-button--insert 'telega qr
                    :inserter 'telega-ins--inline-document
                    :action 'telega-inline-bot--action
-                   'cursor-sensor-functions
-                   '(telega-button-highlight--sensor-func))
+                   :telega-add-sensor-func
+                   #'telega-button-highlight--sensor-func)
                  (telega-ins--inline-delim))
 
                 (inlineQueryResultVideo
                  (telega-button--insert 'telega qr
                    :inserter 'telega-ins--inline-video
                    :action 'telega-inline-bot--action
-                   'cursor-sensor-functions
-                   '(telega-button-highlight--sensor-func))
+                   :telega-add-sensor-func
+                   #'telega-button-highlight--sensor-func)
                  (telega-ins--inline-delim))
 
                 (inlineQueryResultAudio
                  (telega-button--insert 'telega qr
                    :inserter 'telega-ins--inline-audio
                    :action 'telega-inline-bot--action
-                   'cursor-sensor-functions
-                   '(telega-button-highlight--sensor-func))
+                   :telega-add-sensor-func
+                   #'telega-button-highlight--sensor-func)
                  (telega-ins--inline-delim))
 
                 (inlineQueryResultVoiceNote
                  (telega-button--insert 'telega qr
                    :inserter 'telega-ins--inline-voice-note
                    :action 'telega-inline-bot--action
-                   'cursor-sensor-functions
-                   '(telega-button-highlight--sensor-func))
+                   :telega-add-sensor-func
+                   #'telega-button-highlight--sensor-func)
                  (telega-ins--inline-delim))
 
                 (inlineQueryResultArticle
                  (telega-button--insert 'telega qr
                    :inserter 'telega-ins--inline-article
                    :action 'telega-inline-bot--action
-                   'cursor-sensor-functions
-                   '(telega-button-highlight--sensor-func))
+                   :telega-add-sensor-func
+                   'telega-button-highlight--sensor-func)
                  (telega-ins--inline-delim))
 
                 (inlineQueryResultAnimation
                  (telega-button--insert 'telega qr
                    :inserter 'telega-ins--inline-animation
                    :action 'telega-inline-bot--action
-                   'cursor-sensor-functions
-                   (list (telega-animation--gen-sensor-func
-                          (plist-get qr :animation)))
+                   :telega-add-sensor-func
+                   (telega-animation--gen-sensor-func (plist-get qr :animation))
                    'help-echo (when-let ((title (telega-tl-str qr :title)))
                                 (format "GIF title: %s" title))))
 
@@ -486,7 +559,8 @@
       ;; Not found
       (unless (string-empty-p query)
         (message "telega: @%s Nothing found for %s"
-                 (plist-get bot :username) (propertize query 'face 'bold)))
+                 (telega-msg-sender-username bot)
+                 (propertize query 'face 'bold)))
       )))
 
 (defun telega-inline-bot-query (bot query for-chat)
@@ -497,11 +571,20 @@
       (telega-server--callback-put telega-chatbuf--inline-query 'ignore))
 
     (message "telega: @%s Searching for %s..."
-             (telega-tl-str bot :username) (propertize query 'face 'bold))
+             (telega-msg-sender-username bot) (propertize query 'face 'bold))
     (setq telega-chatbuf--inline-query
           (telega--getInlineQueryResults bot query for-chat nil nil
             (telega-inline-bot--gen-callback bot query for-chat)))))
 
+(defun telega--recent-inline-bots-fetch ()
+  "Update recently used bots."
+  (telega--getRecentInlineBots
+   (lambda (users)
+     (setq telega--recent-inline-bots
+           (mapcar (lambda (user)
+                     (telega-user-title user 'username))
+                   users)))))
+
 (provide 'telega-inline)
 
 ;;; telega-inline.el ends here
