@@ -29,13 +29,13 @@
 ;;; Code:
 (require 'cl-lib)
 (require 'url-util)
-(require 'browse-url)                   ; browse-url-url-at-point
 (require 'visual-fill-column)
 
-(require 'telega-server)
+(require 'telega-tdlib)
 (require 'telega-ins)
 (require 'telega-media)
 (require 'telega-tme)
+(require 'telega-sticker)
 (require 'telega-customize)
 
 (defvar telega-webpage-history nil
@@ -100,25 +100,6 @@
       (error "No previous webpage in history"))
     (telega-webpage--history-show idx)))
 
-(defun telega-msg-button--iv-action (button)
-  "Open instant view when BUTTON is pressed."
-  (let* ((msg (button-get button :value))
-         (web-page (telega--tl-get msg :content :web_page)))
-    (funcall 'telega-webpage--instant-view (plist-get web-page :url)
-             (plist-get web-page :site_name))))
-
-(defun telega--getWebPageInstantView (url &optional partial)
-  "Return instant view for the URL.
-Return nil if URL is not available for instant view."
-  (let ((reply (telega-server--call
-                (list :@type "getWebPageInstantView"
-                      :url url
-                      :force_full (or (not partial) :false)))))
-    ;; NOTE: May result in 404 error, return nil in this case
-    (and reply
-         (eq (telega--tl-type reply) 'webPageInstantView)
-         reply)))
-
 (defvar telega-webpage-mode-map
   (let ((map (make-sparse-keymap)))
     (define-key map "g" 'telega-webpage-browse-url)
@@ -137,7 +118,11 @@ Return nil if URL is not available for instant view."
 Keymap:
 \\{telega-webpage-mode-map}"
   :group 'telega
+  ;; Same as for `telega-chat-mode'
+  ;; See https://github.com/zevlg/telega.el/issues/347
+  (setq line-spacing 0)
   (setq-local nobreak-char-display nil)
+
   (setq header-line-format telega-webpage-header-line-format)
   (set-buffer-modified-p nil))
 
@@ -154,15 +139,16 @@ Keymap:
 
 (defun telega-webpage-goto-anchor (name)
   "Goto at anchor NAME position."
-  (let ((anchor (cdr (assoc name telega-webpage--anchors))))
+  ;; NOTE: NAME could be a hex encoded utf-8 string
+  (let* ((dec-name (decode-coding-string (url-unhex-string name) 'utf-8))
+         (anchor (cdr (assoc dec-name telega-webpage--anchors))))
     (unless anchor
-      (error (format "Anchor \"#%s\" not found" name)))
+      (error (format "Anchor \"#%s\" not found" dec-name)))
     (goto-char anchor)))
 
 (defun telega-webpage--ins-pb-details (pb)
   "Inserter for `pageBlockDetails' page block PB."
-  (let ((open-p (not (plist-get pb :is_closed)))
-        (telega--current-buffer (current-buffer)))
+  (let ((open-p (not (plist-get pb :is_closed))))
     (telega-ins (funcall (if open-p 'cdr 'car)
                          telega-symbol-webpage-details)
                 " ")
@@ -176,29 +162,29 @@ Keymap:
       (mapc 'telega-webpage--ins-pb (plist-get pb :page_blocks)))
     ))
 
-(defun telega-webpage-rticon--image (rt limits)
+(defun telega-webpage-rticon--image (rt)
   "Return image representing rich text icon for RT."
   (let* ((doc (plist-get rt :document))
          (width (plist-get rt :width))
          (height (plist-get rt :height))
-         (cheight (telega-media--cheight-for-limits width height limits))
+         (cheight (telega-media--cheight-for-limits
+                   width height
+                   (list 1 1 (nth 2 telega-webpage-photo-size-limits)
+                         (nth 3 telega-webpage-photo-size-limits))))
          (create-image-fun
-          (progn
-            (cl-assert (<= cheight (nth 3 limits)))
-            (lambda (_rtignored &optional _fileignored)
-              ;; 1) FILE downloaded, show FILE
-              ;; 2) Thumbnail is downloaded, use it
-              ;; 3) FILE downloading, fallback to progress svg
-              (let ((doc-file (telega-file--renew doc :document)))
-                (if (telega-file--downloaded-p doc-file)
-                    (telega-media--create-image doc-file width height cheight)
-                  (let* ((thumb (plist-get doc :thumbnail))
-                         (thumb-file (telega-file--renew thumb :photo)))
-                    (if (telega-file--downloaded-p thumb-file)
-                        (telega-thumb--create-image thumb thumb-file cheight)
-                      (telega-media--progress-svg
-                       doc-file width height cheight)))))))))
-
+          (lambda (_rtignored &optional _fileignored)
+            ;; 1) FILE downloaded, show FILE
+            ;; 2) Thumbnail is downloaded, use it
+            ;; 3) FILE downloading, fallback to progress svg
+            (let ((doc-file (telega-file--renew doc :document)))
+              (if (telega-file--downloaded-p doc-file)
+                  (telega-media--create-image doc-file width height cheight)
+                (let* ((thumb (plist-get doc :thumbnail))
+                       (thumb-file (telega-file--renew thumb :photo)))
+                  (if (telega-file--downloaded-p thumb-file)
+                      (telega-thumb--create-image thumb thumb-file cheight)
+                    (telega-media--progress-svg
+                     doc-file width height cheight))))))))
     (telega-media--image
      (cons rt create-image-fun)
      (cons doc :document)
@@ -207,40 +193,36 @@ Keymap:
 (defun telega-webpage--add-anchor (name)
   "Add anchor with the NAME to point at POS position."
   (setf (alist-get name telega-webpage--anchors nil nil 'equal)
-        (with-current-buffer telega--current-buffer
-          (point-marker))))
+        (point-marker)))
 
 (defun telega-webpage--ins-rt (rt)
   "Insert RichText RT."
   (cl-ecase (telega--tl-type rt)
     (richTextAnchor
-     (telega-webpage--add-anchor (plist-get rt :name))
-     (when-let ((anchor-rt (plist-get rt :text)))
-       ;; NOTE: compatibility with TDLib 1.6.0
-       (telega-webpage--ins-rt anchor-rt)))
+     (telega-webpage--add-anchor (telega-tl-str rt :name)))
     (richTextReference
      ;; TDLib 1.6.2
-     (let ((ref-url (plist-get rt :url)))
+     (let ((ref-url (telega-tl-str rt :url)))
        (telega-ins--raw-button
-           (list 'face 'telega-link
-                 'action #'telega-button--action
+           (list 'action #'telega-button--action
                  :help-echo (concat "Reference: " ref-url)
                  :value ref-url
                  :action #'telega-browse-url)
-         (telega-webpage--ins-rt (plist-get rt :text)))))
+         (telega-ins--with-face 'telega-link
+           (telega-webpage--ins-rt (plist-get rt :text))))))
     (richTextAnchorLink
      ;; TDLib 1.6.2
-     (let ((anchor (plist-get rt :name)))
+     (let ((anchor (telega-tl-str rt :anchor_name)))
        (telega-ins--raw-button
-           (list 'face 'telega-link
-                 'action #'telega-button--action
+           (list 'action #'telega-button--action
                  :help-echo (concat "Anchor: #" anchor)
                  :value anchor
                  :action #'telega-webpage-goto-anchor)
-       (telega-webpage--ins-rt (plist-get rt :text)))))
+         (telega-ins--with-face 'telega-link
+           (telega-webpage--ins-rt (plist-get rt :text))))))
     (richTextIcon
      (telega-ins--image-slices
-      (telega-webpage-rticon--image rt telega-photo-size-limits)))
+      (telega-webpage-rticon--image rt)))
     (richTextPlain
      (telega-ins (funcall (if telega-webpage-strip-nl
                               #'telega-strip-newlines
@@ -249,31 +231,31 @@ Keymap:
     (richTexts
      (mapc #'telega-webpage--ins-rt (plist-get rt :texts)))
     (richTextBold
-     (telega-ins--with-attrs (list :face 'bold)
+     (telega-ins--with-face 'bold
        (telega-webpage--ins-rt (plist-get rt :text))))
     (richTextItalic
-     (telega-ins--with-attrs (list :face 'italic)
+     (telega-ins--with-face 'italic
        (telega-webpage--ins-rt (plist-get rt :text))))
     (richTextUnderline
-     (telega-ins--with-attrs (list :face 'underline)
+     (telega-ins--with-face 'underline
        (telega-webpage--ins-rt (plist-get rt :text))))
     (richTextStrikethrough
-     (telega-ins--with-attrs (list :face 'telega-webpage-strike-through)
+     (telega-ins--with-face 'telega-webpage-strike-through
        (telega-webpage--ins-rt (plist-get rt :text))))
     (richTextFixed
-     (telega-ins--with-attrs (list :face 'telega-webpage-fixed)
+     (telega-ins--with-face 'telega-webpage-fixed
        (telega-webpage--ins-rt (plist-get rt :text))))
     (richTextUrl
-     (let ((url (plist-get rt :url)))
+     (let ((url (telega-tl-str rt :url)))
        (telega-ins--raw-button
-           (list 'face 'telega-link
-                 'action #'telega-button--action
+           (list 'action #'telega-button--action
                  :help-echo (concat "URL: " url)
                  :value url
                  :action #'telega-browse-url)
-         (telega-webpage--ins-rt (plist-get rt :text)))))
+         (telega-ins--with-face 'telega-link
+           (telega-webpage--ins-rt (plist-get rt :text))))))
     (richTextEmailAddress
-     (telega-ins--with-attrs (list :face 'link)
+     (telega-ins--with-face 'telega-link
        (telega-webpage--ins-rt (plist-get rt :text))))
     (richTextSubscript
      (telega-ins--with-props '(display (raise -0.25))
@@ -284,7 +266,7 @@ Keymap:
        (telega-ins--with-face '(:height 0.5)
          (telega-webpage--ins-rt (plist-get rt :text)))))
     (richTextMarked
-     (telega-ins--with-attrs (list :face 'region)
+     (telega-ins--with-face 'telega-webpage-marked
        (telega-webpage--ins-rt (plist-get rt :text))))
     ))
 
@@ -293,8 +275,9 @@ Keymap:
   (let* ((pb-photo (plist-get pageblock :photo))
          (photo-image (when pb-photo
                         (telega-photo--image pb-photo (list 10 3 10 3))))
-         (url (plist-get pageblock :url))
-         (title (plist-get pageblock :title))
+         (url (telega-tl-str pageblock :url))
+         (title (telega-tl-str pageblock :title))
+         (desc (telega-tl-str pageblock :description))
          (author (plist-get pageblock :author))
          (publish-date (plist-get pageblock :publish_date)))
     (telega-ins--with-attrs (list :max telega-webpage-fill-column
@@ -308,20 +291,73 @@ Keymap:
       (telega-ins "\n")
       (when photo-image
         (telega-ins--image photo-image 1))
-      (telega-ins--with-face 'shadow
+      (telega-ins--with-face 'telega-shadow
         (if author
             (telega-ins author)
           (telega-ins "Unknown author"))
         (when (zerop publish-date)
           (setq publish-date (time-to-seconds)))
         (telega-ins " • ")
-        (telega-ins--date-full publish-date))
+        (telega-ins--date publish-date 'date-long))
       (telega-ins "\n")
       (when photo-image
         (telega-ins--image photo-image 2))
       (telega-ins--with-face 'telega-link
         (telega-ins url)))))
 
+(defun telega-webpage--animation-ffplay-callback (proc frame anim)
+  "Callback for inline animation playback."
+  (let ((telega-animation-height (telega-media--cheight-for-limits
+                                  (plist-get anim :width)
+                                  (plist-get anim :height)
+                                  telega-webpage-photo-size-limits)))
+    (telega-animation--ffplay-callback proc frame anim)))
+
+(defun telega-webpage--animation-sensor-func (_window oldpos dir)
+  "Sensor function for the animation to start/stop playing."
+  (let* ((pos (if (eq dir 'entered) (point) oldpos))
+         (anim (get-text-property pos :telega-animation)))
+    (if (eq dir 'entered)
+        (telega-file--download (telega-file--renew anim :animation)
+          :priority 32
+          :update-callback
+          (lambda (file)
+            (when (telega-file--downloaded-p file)
+              (telega-ffplay-to-png (telega--tl-get file :local :path) "-an"
+                (list #'telega-webpage--animation-ffplay-callback anim)))))
+
+      (telega-ffplay-stop)
+      (telega-file--cancel-download (plist-get anim :animation)))))
+
+(defun telega-webpage--animation-inserter (anim)
+  "Inserter for the animation button."
+  (let ((telega-animation-height (telega-media--cheight-for-limits
+                                  (plist-get anim :width)
+                                  (plist-get anim :height)
+                                  telega-webpage-photo-size-limits)))
+    (telega-ins--animation-image anim 'sliced)))
+
+(defun telega-webpage--animation-action (anim)
+  "Action to take when animation button in pressed."
+  (telega-ffplay-stop)
+  (telega-file--cancel-download (plist-get anim :animation) 'sync)
+
+  (telega-file--download (telega-file--renew anim :animation)
+    :priority 32
+    :update-callback
+    (lambda (afile)
+      (when (telega-file--downloaded-p afile)
+        (telega-video-player-run (telega--tl-get afile :local :path))))))
+
+(defun telega-webpage--ins-animation (animation)
+  "Insert pageblock with ANIMATION."
+  (telega-button--insert 'telega animation
+    :inserter #'telega-webpage--animation-inserter
+    :action #'telega-webpage--animation-action
+    :telega-animation animation
+    :telega-add-sensor-func #'telega-webpage--animation-sensor-func
+    'help-echo "RET to play animation"))
+
 (defun telega-webpage--ins-pb (pb)
   "Insert PageBlock PB for the instant view."
   (cl-ecase (telega--tl-type pb)
@@ -329,34 +365,39 @@ Keymap:
      (telega-ins--with-face 'telega-webpage-title
        (telega-webpage--ins-rt (plist-get pb :title))))
     (pageBlockSubtitle
-     (telega-webpage--ins-rt (plist-get pb :subtitle)))
+     (telega-ins--with-face 'telega-webpage-subtitle
+       (telega-webpage--ins-rt (plist-get pb :subtitle))))
     (pageBlockAuthorDate
-     (telega-ins--with-attrs (list :face 'shadow)
-       (telega-ins "By ")
-       (telega-webpage--ins-rt (plist-get pb :author))
-       (telega-ins " • ")
+     (telega-ins--with-face 'telega-shadow
+       (telega-ins-prefix "By "
+         (when (telega-webpage--ins-rt (plist-get pb :author))
+           (telega-ins " • ")))
        (let ((publish-date (plist-get pb :publish_date)))
          (when (zerop publish-date)
            (setq publish-date (time-to-seconds)))
-         (telega-ins--date-full publish-date)))
+         (telega-ins--date publish-date 'date-long)))
      (telega-ins "\n"))
     (pageBlockHeader
+     (telega-ins "\n")
      (telega-ins--with-face 'telega-webpage-header
        (telega-webpage--ins-rt (plist-get pb :header))
        (telega-ins "\n")))
     (pageBlockSubheader
+     (telega-ins "\n")
      (telega-ins--with-face 'telega-webpage-subheader
        (telega-webpage--ins-rt (plist-get pb :subheader))
        (telega-ins "\n")))
     (pageBlockParagraph
-     (telega-webpage--ins-rt (plist-get pb :text))
-     (telega-ins "\n"))
+     (unless (looking-back "\n\n" nil)
+       (telega-ins "\n"))
+     (telega-webpage--ins-rt (plist-get pb :text)))
     (pageBlockPreformatted
+     (telega-ins "\n")
      (telega-ins--with-face 'telega-webpage-preformatted
        (telega-webpage--ins-rt (plist-get pb :text))
        (telega-ins "\n")))
     (pageBlockFooter
-     (telega-ins--with-face 'shadow
+     (telega-ins--with-face 'telega-shadow
        (telega-ins (make-string (/ telega-webpage-fill-column 2) ?-) "\n")
        (telega-webpage--ins-rt (plist-get pb :footer))))
     (pageBlockDivider
@@ -365,32 +406,46 @@ Keymap:
     (pageBlockAnchor
      (telega-webpage--add-anchor (plist-get pb :name)))
     (pageBlockListItem
-     (telega-ins--labeled (concat " " (plist-get pb :label) " ") nil
-       telega-webpage-fill-column
-       (mapc 'telega-webpage--ins-pb (plist-get pb :page_blocks))))
+     (telega-ins-from-newline
+      (let ((label (or (telega-tl-str pb :label) "•")))
+        (telega-ins--line-wrap-prefix
+            (cons (concat (propertize label 'face 'bold) " ")
+                  (concat (make-string (string-width label) ?\s " ") " "))
+          (telega-ins (telega-strip-newlines
+                       (telega-ins--as-string
+                        (mapc #'telega-webpage--ins-pb
+                              (plist-get pb :page_blocks)))))))))
     (pageBlockList
      (let ((telega-webpage-strip-nl t))
        (mapc 'telega-webpage--ins-pb (plist-get pb :items))))
     (pageBlockBlockQuote
-     (let ((vbar-prefix (propertize (telega-symbol 'vertical-bar) 'face 'bold)))
+     (telega-ins "\n")
+     (let ((vbar-prefix (propertize (telega-symbol 'vbar-left) 'face 'bold)))
        (telega-ins vbar-prefix)
        (telega-ins--with-attrs (list :fill 'left
                                      :fill-column telega-webpage-fill-column
                                      :fill-prefix vbar-prefix)
-         (telega-webpage--ins-rt (plist-get pb :text)))
+         (telega-webpage--ins-rt (plist-get pb :text))
+         (when-let ((credit-rt (plist-get pb :credit)))
+           (telega-ins "\n")
+           (telega-ins--with-face 'telega-shadow
+             (telega-webpage--ins-rt credit-rt))))
        (telega-ins "\n")))
     (pageBlockPullQuote
-     (telega-ins "\u00A0\u00A0")
-     (telega-ins--with-attrs (list :fill 'center
-                                   :fill-column (- telega-webpage-fill-column 2)
-                                   :fill-prefix "\u00A0\u00A0")
-       (telega-webpage--ins-rt (plist-get pb :text)))
-     (telega-ins "\n"))
+     (telega-ins-from-newline
+      (telega-ins "\u00A0\u00A0")
+      (telega-ins--with-attrs (list :fill 'center
+                                    :fill-column (- telega-webpage-fill-column 2)
+                                    :fill-prefix "\u00A0\u00A0")
+        (telega-webpage--ins-rt (plist-get pb :text)))
+      (telega-ins "\n")))
     (pageBlockAnimation
-     (telega-ins "<TODO: pageBlockAnimation>\n"))
+     (telega-webpage--ins-animation (plist-get pb :animation))
+     (telega-ins "\n"))
     (pageBlockAudio
      (telega-ins "<TODO: pageBlockAudio>\n"))
     (pageBlockPhoto
+     (telega-ins "\n")
      (telega-button--insert 'telega (plist-get pb :photo)
        :inserter (lambda (photo)
                    (telega-ins--photo photo nil telega-webpage-photo-size-limits))
@@ -405,7 +460,7 @@ Keymap:
     (pageBlockEmbedded
      (telega-button--insert 'telega pb
        :inserter (lambda (pb-embedded)
-                   (let ((url (plist-get pb-embedded :url))
+                   (let ((url (telega-tl-str pb-embedded :url))
                          (html (plist-get pb-embedded :html))
                          (poster-photo (plist-get pb-embedded :poster_photo)))
                      (when (telega-ins--with-face '(bold telega-link)
@@ -418,14 +473,13 @@ Keymap:
                      (when poster-photo
                        (telega-ins--photo
                         poster-photo nil telega-webpage-photo-size-limits))
-                     (telega-ins-prefix "\n"
+                     (telega-ins-from-newline
                        (telega-webpage--ins-pb
                         (plist-get pb-embedded :caption)))
                      ))
        :action (lambda (pb-embedded)
-                 (let ((url (plist-get pb-embedded :url)))
-                   (unless (string-empty-p url)
-                     (telega-browse-url url))))))
+                 (when-let ((url (telega-tl-str pb-embedded :url)))
+                   (telega-browse-url url)))))
     (pageBlockEmbeddedPost
      (telega-ins "<TODO: pageBlockEmbeddedPost>\n"))
     (pageBlockCollage
@@ -434,18 +488,20 @@ Keymap:
     (pageBlockSlideshow
      (let ((page-blocks (plist-get pb :page_blocks)))
        (dotimes (n (length page-blocks))
-         (telega-ins--labeled (format "%d/%d " (1+ n) (length page-blocks)) nil
-           (telega-webpage--ins-pb (aref page-blocks n)))))
+         (telega-ins-from-newline
+          (telega-webpage--ins-pb (aref page-blocks n))
+          (telega-ins-fmt "%d/%d\n" (1+ n) (length page-blocks)))))
      (telega-webpage--ins-pb (plist-get pb :caption)))
     (pageBlockChatLink
-     (telega-ins--with-attrs (list :face 'telega-webpage-chat-link)
+     (telega-ins--with-face 'telega-webpage-chat-link
        (telega-ins (telega-tl-str pb :title) " "
                    "@" (telega-tl-str pb :username) " ")
-       (telega-ins--button "Open"
+       (telega-ins--box-button (telega-i18n "lng_open_link")
          :value (telega-tl-str pb :username)
-         :action 'telega-tme-open-username)))
+         :action 'telega-tme-open-username)
+       (telega-ins "\n")))
     (pageBlockCaption
-     (telega-ins--with-face 'shadow
+     (telega-ins--with-face 'telega-shadow
        (telega-webpage--ins-rt (plist-get pb :text))
        (telega-ins-prefix " --"
          (telega-webpage--ins-rt (plist-get pb :credit)))))
@@ -468,22 +524,24 @@ Keymap:
      (telega-button--insert 'telega pb
        :inserter 'telega-webpage--ins-related-article
        :action (lambda (_pbignored)
-                 (telega-browse-url (plist-get pb :url)))
-       :help-echo (concat "URL: " (plist-get pb :url))))
+                 (telega-browse-url (telega-tl-str pb :url)))
+       :help-echo (concat "URL: " (telega-tl-str pb :url))))
     (pageBlockRelatedArticles
-     (telega-ins--with-face '(telega-msg-heading bold)
-       (when (telega-webpage--ins-rt (plist-get pb :header))
-         (telega-ins "\n")))
-     (mapc 'telega-webpage--ins-pb (plist-get pb :articles)))
+     (telega-ins--with-face '(telega-webpage-outline bold)
+       (telega-webpage--ins-rt (plist-get pb :header))
+       (telega-ins "\n"))
+     (seq-doseq (article-pb (plist-get pb :articles))
+       (telega-webpage--ins-pb article-pb)
+       (telega-ins "\n")))
     (pageBlockKicker
      (telega-webpage--ins-rt (plist-get pb :kicker)))
     )
 
-  (unless telega-webpage-strip-nl
-    (unless (memq (telega--tl-type pb)
-                  '(pageBlockAnchor pageBlockCover pageBlockListItem
-                                    pageBlockDetails))
-      (telega-ins "\n")))
+  ;; (unless telega-webpage-strip-nl
+  ;;   (unless (memq (telega--tl-type pb)
+  ;;                 '(pageBlockAnchor pageBlockCover pageBlockListItem
+  ;;                                   pageBlockDetails))
+  ;;     (telega-ins "\n")))
   t)
 
 (defun telega-webpage--ins-pb-nl (pb)
@@ -509,29 +567,54 @@ instant view for the URL."
         telega-webpage--iv (or instant-view
                                (telega--getWebPageInstantView url)
                                (error "Can't instant view the URL: %s" url))
-        ;; IV.url can differ from URL, we use IV.url for the correct
-        ;; anchors handling. See docs for "td-api.tl:webPageInstantView"
-        telega-webpage--url (or (plist-get telega-webpage--iv :url) url)
+        telega-webpage--url url
         telega-webpage--anchors nil)
   (telega-webpage--history-push)
 
-  (let ((buffer-read-only nil)
-        (telega--current-buffer (current-buffer)))
+  (with-telega-buffer-modify
     (erase-buffer)
-    (mapc 'telega-webpage--ins-pb
-          (plist-get telega-webpage--iv :page_blocks))
-    (when telega-debug
-      (telega-ins-fmt "\n---DEBUG---\n%S" telega-webpage--iv))
+    (seq-doseq (iv-pb (plist-get telega-webpage--iv :page_blocks))
+      (telega-ins-from-newline
+       (telega-webpage--ins-pb iv-pb)))
+
+    ;; IV status
+    (telega-ins-from-newline
+     (telega-ins "\n")
+     (telega-ins--with-face '(:inherit telega-webpage-outline :overline t)
+       (let ((view-count (plist-get telega-webpage--iv :view_count)))
+         (unless (telega-zerop view-count)
+           (telega-ins-i18n "lng_views_tooltip"
+             :count view-count)))
+       (when-let ((tdlib-link (plist-get telega-webpage--iv :feedback_link))
+                  (link-text (telega-i18n "lng_iv_wrong_layout")))
+         (telega-ins--move-to-column (- telega-webpage-fill-column
+                                        (string-width link-text)))
+         (telega-ins--raw-button (telega-link-props 'tdlib-link tdlib-link
+                                                    'face 'link)
+           (telega-ins link-text)))
+       (telega-ins "\n")))
+
+    (when (and (listp telega-debug) (memq 'iv telega-debug))
+      (telega-ins-from-newline
+       (telega-ins "\n")
+       (telega-ins-describe-section "DEBUG")
+       (telega-ins-fmt "%S" telega-webpage--iv)))
     (goto-char (point-min)))
 
-  (unless (eq major-mode 'telega-webpage-mode)
+  (unless (derived-mode-p 'telega-webpage-mode)
     (telega-webpage-mode)
 
     (cursor-sensor-mode 1)
+    (cursor-face-highlight-mode 1)
     (visual-line-mode 1)
     (setq visual-fill-column-width telega-webpage-fill-column)
     (visual-fill-column-mode 1))
 
+  (when (plist-get telega-webpage--iv :is_rtl)
+    ;; Force RTL display
+    (setq bidi-display-reordering t
+          bidi-paragraph-direction 'right-to-left))
+
   (let ((anchor (string-trim-left url "[^#]*#?")))
     (unless (string-empty-p anchor)
       (telega-webpage-goto-anchor anchor)))
@@ -545,13 +628,27 @@ instant view for the URL."
    (concat (string-trim-right telega-webpage--url "#.*") "#")
    url))
 
+(defun telega-url-at-point ()
+  "Return URL at point.
+If called interactively copies url at point to the kill ring."
+  (interactive)
+  (let* ((button (button-at (point)))
+         (link (when button (button-get button :telega-link)))
+         (url-ap (or (and link (eq (car link) 'url) (cdr link))
+                     (thing-at-point 'url t))))
+    (if (called-interactively-p 'interactive)
+        (if (not url-ap)
+            (user-error "telega: No URL at point")
+          (kill-new url-ap)
+          (message "telega: Copied %S" url-ap))
+      url-ap)))
+
 (defun telega-browse-url (url &optional in-web-browser)
   "Open the URL.
 If URL can be opened directly inside telega, then do it.
 Invite links and link to users can be directly opened in telega.
 If IN-WEB-BROWSER is non-nil then force opening in web browser."
-  (interactive (list (browse-url-url-at-point)
-                     current-prefix-arg))
+  (interactive (list (telega-url-at-point) current-prefix-arg))
   (when (or in-web-browser
             (not (cond ((string-prefix-p "tg:" url)
                         (telega-tme-open-tg url))
@@ -581,6 +678,8 @@ If IN-WEB-BROWSER is non-nil then force opening in web browser."
         ;; Fallback to default
         (browse-url url)))))
 
+;;; webview for webapp using xwidgets
+
 (provide 'telega-webpage)
 
 ;;; telega-webpage.el ends here
